@@ -20,11 +20,11 @@
  * @author Sam Lang, Sam Meder
  * 
  * $RCSfile: globus_i_gsi_gss_utils.c,v $
- * $Revision: 1.47.2.4 $
- * $Date: 2008/09/24 20:26:34 $
+ * $Revision: 1.55.2.5 $
+ * $Date: 2010/03/26 16:11:03 $
  */
 
-static char *rcsid = "$Id: globus_i_gsi_gss_utils.c,v 1.47.2.4 2008/09/24 20:26:34 bester Exp $";
+static char *rcsid = "$Id: globus_i_gsi_gss_utils.c,v 1.55.2.5 2010/03/26 16:11:03 bester Exp $";
 
 #include "gssapi_openssl.h"
 #include "globus_i_gsi_gss_utils.h"
@@ -37,7 +37,7 @@ static char *rcsid = "$Id: globus_i_gsi_gss_utils.c,v 1.47.2.4 2008/09/24 20:26:
 #include <string.h>
 #include <stdlib.h>
 
-#include "ssl_locl.h"
+#include "globus_ssl_locl.h"
 
 #ifdef WIN32
 #define strcasecmp stricmp
@@ -270,7 +270,7 @@ globus_i_gsi_gss_create_and_fill_context(
             goto exit;
         }
         
-        memset(context, (int)NULL, sizeof(gss_ctx_id_desc));
+        memset(context, 0, sizeof(gss_ctx_id_desc));
         *context_handle_P = context;
         context->ctx_flags = 0;
     }
@@ -295,7 +295,7 @@ globus_i_gsi_gss_create_and_fill_context(
         goto free_context;
     }
 
-    memset(context->peer_cred_handle, (int) NULL, sizeof(gss_cred_id_desc));
+    memset(context->peer_cred_handle, 0, sizeof(gss_cred_id_desc));
     
     local_result = globus_gsi_cred_handle_init(
         &context->peer_cred_handle->cred_handle, NULL);
@@ -430,6 +430,27 @@ globus_i_gsi_gss_create_and_fill_context(
         free(certdir);
         certdir = NULL;
     }
+    #if (OPENSSL_VERSION_NUMBER >= 0x009080dfL)
+    {
+        /*
+         * OpenSSL 0.9.8m and 1.0.0 introduce changes to how
+         * ssl3_output_cert_chain creates the certificate chain to send.
+         *
+         * The new code uses X509_verify_cert(), which fails if
+         * a certificate does not have a X509_V_ERR_KEYUSAGE_NO_CERTSIGN but
+         * signs a proxy. As a result, the entire certificate chain is not
+         * sent during the handshake.
+         *
+         * This code causes the issuer checks to use
+         * globus_gsi_callback_check_issued() to handle that error if
+         * the certificate in question is a proxy.
+         */
+        X509_STORE * store;
+
+        store = SSL_CTX_get_cert_store(context->cred_handle->ssl_context);
+        store->check_issued = globus_gsi_callback_check_issued;
+    }
+    #endif
 
     context->gss_ssl = SSL_new(context->cred_handle->ssl_context);
 
@@ -564,17 +585,17 @@ globus_i_gsi_gss_create_and_fill_context(
     {
         char buff[256];
         int i;
-        STACK *sk;
+        STACK_OF(SSL_CIPHER) *sk;
         
         GLOBUS_I_GSI_GSSAPI_DEBUG_PRINT(
             2, "Ciphers available:\n");
-        sk=(STACK *)SSL_get_ciphers(context->gss_ssl);
-        for (i=0; i<sk_num(sk); i++)
+        sk=SSL_get_ciphers(context->gss_ssl);
+        for (i=0; i<sk_SSL_CIPHER_num(sk); i++)
         {
-            SSL_CIPHER_description((SSL_CIPHER *)sk_value(sk,i),
+            SSL_CIPHER_description(sk_SSL_CIPHER_value(sk,i),
                                    buff,256);
             GLOBUS_I_GSI_GSSAPI_DEBUG_FPRINTF(
-                3, (globus_i_gsi_gssapi_debug_fstream, buff));
+                3, (globus_i_gsi_gssapi_debug_fstream, "%s", buff));
         }
     }
 
@@ -995,7 +1016,7 @@ globus_i_gsi_gss_handshake(
     if (!GSS_ERROR(major_status)) {
         if (rc > 0)
         {
-            SSL_CIPHER *                current_cipher;
+            const SSL_CIPHER *              current_cipher;
             major_status = GSS_S_COMPLETE; 
 
             /*
@@ -1423,110 +1444,6 @@ globus_i_gsi_gss_cred_read(
 }
 /* @} */
 
-/**
- * @name Credential Set
- * @ingroup globus_i_gsi_gss_utils
- */
-/* @{ */
-/**
- * Credential Set
- *
- * @param minor_status
- * @param cred_usage
- * @param cred_handle
- * @param ucert
- * @param upkey,
- * @param cert_chain
- *
- * @return
- */
-OM_uint32
-globus_i_gsi_gss_cred_set(
-    OM_uint32 *                         minor_status,
-    const gss_cred_usage_t              cred_usage,
-    gss_cred_id_t *                     cred_handle,
-    X509 *                              ucert,
-    EVP_PKEY *                          upkey,
-    STACK_OF(X509) *                    cert_chain)
-{
-    OM_uint32                           major_status = GSS_S_COMPLETE;
-    OM_uint32                           local_minor_status;
-    globus_result_t                     local_result;
-    globus_gsi_cred_handle_t            local_cred_handle;
-    static char *                       _function_name_ =
-        "globus_i_gsi_gss_cred_set";
-
-    GLOBUS_I_GSI_GSSAPI_DEBUG_ENTER;
-
-    *minor_status = GLOBUS_SUCCESS;
-
-    local_result = globus_gsi_cred_handle_init(&local_cred_handle, NULL);
-    if(local_result != GLOBUS_SUCCESS)
-    {
-        local_cred_handle = NULL;
-        GLOBUS_GSI_GSSAPI_ERROR_CHAIN_RESULT(
-            minor_status, local_result,
-            GLOBUS_GSI_GSSAPI_ERROR_WITH_GSI_CREDENTIAL);
-        major_status = GSS_S_FAILURE;
-        goto exit;
-    }
-
-    local_result = globus_gsi_cred_set_cert(local_cred_handle, ucert);
-    if(local_result != GLOBUS_SUCCESS)
-    {
-        GLOBUS_GSI_GSSAPI_ERROR_CHAIN_RESULT(
-            minor_status, local_result,
-            GLOBUS_GSI_GSSAPI_ERROR_WITH_GSI_CREDENTIAL);
-        major_status = GSS_S_FAILURE;
-        goto exit;
-    }
-    
-    local_result = globus_gsi_cred_set_key(local_cred_handle, upkey);
-    if(local_result != GLOBUS_SUCCESS)
-    {
-        GLOBUS_GSI_GSSAPI_ERROR_CHAIN_RESULT(
-            minor_status, local_result,
-            GLOBUS_GSI_GSSAPI_ERROR_WITH_GSI_CREDENTIAL);
-        major_status = GSS_S_FAILURE;
-        goto exit;
-    }
-
-    local_result = globus_gsi_cred_set_cert_chain(local_cred_handle, 
-                                                  cert_chain);
-    if(local_result != GLOBUS_SUCCESS)
-    {
-        GLOBUS_GSI_GSSAPI_ERROR_CHAIN_RESULT(
-            minor_status, local_result,
-            GLOBUS_GSI_GSSAPI_ERROR_WITH_GSI_CREDENTIAL);
-        major_status = GSS_S_FAILURE;
-        goto exit;
-    }
-
-    major_status = globus_i_gsi_gss_create_cred(&local_minor_status,
-                                                cred_usage,
-                                                cred_handle, 
-                                                &local_cred_handle);
-    if(GSS_ERROR(major_status))
-    {
-        GLOBUS_GSI_GSSAPI_ERROR_CHAIN_RESULT(
-            minor_status, local_minor_status,
-            GLOBUS_GSI_GSSAPI_ERROR_WITH_GSS_CREDENTIAL);
-        major_status = GSS_S_FAILURE;
-        goto exit;
-    }
-    
- exit:
-
-    if(local_cred_handle != NULL)
-    {
-        globus_gsi_cred_handle_destroy(local_cred_handle);
-    }
-    
-    GLOBUS_I_GSI_GSSAPI_DEBUG_EXIT;
-    return major_status;
-}
-/* @} */
-
 /**
  * @name Create Cred
  * @ingroup globus_i_gsi_gss_utils
@@ -1561,6 +1478,7 @@ globus_i_gsi_gss_create_cred(
     globus_gsi_cert_utils_cert_type_t   cert_type;
     gss_buffer_desc                     name_buffer;
     X509 *                              eec;
+    STACK_OF(X509) *                    cert_chain = NULL;
     globus_bool_t                       free_eec = GLOBUS_FALSE;
     
     static char *                       _function_name_ =
@@ -1623,8 +1541,6 @@ globus_i_gsi_gss_create_cred(
 
     if(GLOBUS_GSI_CERT_UTILS_IS_PROXY(cert_type))
     {
-        STACK_OF(X509) *                cert_chain;
-        
         local_result = globus_gsi_cred_get_cert_chain(
             newcred->cred_handle, 
             &cert_chain);
@@ -1646,6 +1562,7 @@ globus_i_gsi_gss_create_cred(
                 GLOBUS_GSI_GSSAPI_ERROR_WITH_GSI_CREDENTIAL);
             major_status = GSS_S_FAILURE;
             sk_X509_pop_free(cert_chain, X509_free);
+            cert_chain = NULL;
             goto error_exit;
         }
     }
@@ -1698,6 +1615,10 @@ globus_i_gsi_gss_create_cred(
     {
         X509_free(eec);
     }
+    if (cert_chain != NULL)
+    {
+        sk_X509_pop_free(cert_chain, X509_free);
+    }
     GLOBUS_I_GSI_GSSAPI_DEBUG_ENTER;
     return major_status;
 }
@@ -1957,6 +1878,9 @@ globus_i_gsi_gss_SSL_read_bio(
         ssl_handle->session,
         &ssl_handle->s3->tmp.new_sym_enc,
         &ssl_handle->s3->tmp.new_hash,
+#if (OPENSSL_VERSION_NUMBER >= 0x10000000L)
+        NULL, NULL,
+#endif
         (SSL_COMP **) &ssl_handle->s3->tmp.new_compression);
     if (!ssl_result)
     {
@@ -1985,6 +1909,9 @@ globus_i_gsi_gss_SSL_read_bio(
         ssl_handle->session,
         &ssl_handle->s3->tmp.new_sym_enc,
         &ssl_handle->s3->tmp.new_hash,
+#if (OPENSSL_VERSION_NUMBER >= 0x10000000L)
+        NULL, NULL,
+#endif
         (SSL_COMP **) &ssl_handle->s3->tmp.new_compression);
     if (!ssl_result)
     {
@@ -2149,14 +2076,6 @@ globus_i_gsi_gssapi_init_ssl_context(
         goto exit;
     }
 
-    /*
-     * post-0.9.8 versions of the SSL library seem to move part of the 
-     * cipher setup code into SSL_library_init(). Without this call, the
-     * SSL_CTX_new routine comaplains at not being able to initialize the 
-     * list of ciphers.
-     */
-    SSL_library_init();
-
 #if (OPENSSL_VERSION_NUMBER >= 0x00908000L) && !defined(OPENSSL_NO_COMP)
     /*
      * post-0.9.8 versions of OpenSSL include data compression. unfortunately,
@@ -2475,7 +2394,7 @@ int globus_i_gsi_gss_verify_extensions_callback(
     
     local_result = globus_gsi_callback_get_extension_oids(
         callback_data,
-        (void **) &extension_oids);
+        (void **) (void *) &extension_oids);
 
     if(local_result != GLOBUS_SUCCESS)
     {
@@ -2511,16 +2430,22 @@ int globus_i_gsi_gss_verify_extensions_callback(
 }
 /* @} */
 
-unsigned char *
+OM_uint32
 globus_i_gsi_gssapi_get_hostname(
-    const gss_name_desc *               name)
+    OM_uint32 *                         minor_status,
+    gss_name_desc *                     name)
 {
+    OM_uint32                           major_status = GSS_S_COMPLETE;
     int                                 common_name_NID;
     int                                 index;
     unsigned int                        length;
     unsigned char *                     data;
-    unsigned char *                     result = NULL;
+    unsigned char *                     p;
     X509_NAME_ENTRY *                   name_entry = NULL;
+    static const char * _function_name_ = "globus_i_gsi_gssapi_get_hostname";
+
+    name->service_name = name->host_name = NULL;
+    *minor_status = GLOBUS_SUCCESS;
 
     common_name_NID = OBJ_txt2nid("CN");
     for (index = 0; index < X509_NAME_entry_count(name->x509n); index++)
@@ -2530,36 +2455,82 @@ globus_i_gsi_gssapi_get_hostname(
         {
             length = name_entry->value->length;
             data = name_entry->value->data;
-            if ( length > 5 && !strncasecmp((char *) data, "host/", 5))
+
+            p = memchr(data, '/', length);
+
+            if (p)
             {
-                length -= 5;
-                data += 5;
+                name->service_name = malloc(p-data+1);
+                if (name->service_name == NULL)
+                {
+                    GLOBUS_GSI_GSSAPI_MALLOC_ERROR(minor_status);
+                    major_status = GSS_S_FAILURE;
+
+                    goto out;
+                }
+                strncpy(name->service_name, (char *) data, p-data);
+                name->service_name[p-data] = 0;
+
+                name->host_name = malloc(length - (p-data));
+                if (name->host_name == NULL)
+                {
+                    GLOBUS_GSI_GSSAPI_MALLOC_ERROR(minor_status);
+                    major_status = GSS_S_FAILURE;
+
+                    goto free_service_name_out;
+                }
+                strncpy(name->host_name, (char *) p+1, length - (p+1-data));
+                name->host_name[length - (p+1-data)] = 0;
             }
-            else if ( length > 4 && 
-                      !strncasecmp((char *) data, "ftp/", 4))
+            else
             {
-                length -= 4;
-                data += 4;
+                if (gss_i_name_compatibility_mode ==
+                    GSS_I_COMPATIBILITY_STRICT_RFC2818)
+                {
+                    name->service_name = globus_libc_strdup("host");
+                    if (name->service_name == NULL)
+                    {
+                        GLOBUS_GSI_GSSAPI_MALLOC_ERROR(minor_status);
+                        major_status = GSS_S_FAILURE;
+
+                        goto out;
+                    }
+                }
+
+                name->host_name = malloc(length + 1);
+                if (name->host_name == NULL)
+                {
+                    GLOBUS_GSI_GSSAPI_MALLOC_ERROR(minor_status);
+                    major_status = GSS_S_FAILURE;
+
+                    goto free_service_name_out;
+                }
+
+                strncpy(name->host_name, (char *) data, length);
+                name->host_name[length] = 0;
             }
             break;
         }
-        name_entry = NULL;
     }
 
-    if(name_entry)
-    { 
-        result = malloc(length + 1);
-        
-        if(result == NULL)
+    if (name->host_name == NULL)
+    {
+        GLOBUS_GSI_GSSAPI_ERROR_RESULT(
+                minor_status,
+                GLOBUS_GSI_GSSAPI_ERROR_BAD_NAME,
+                ("No common name in subject"));
+        major_status = GSS_S_FAILURE;
+
+free_service_name_out:
+        if (name->service_name)
         {
-            return result;
+            free(name->service_name);
+            name->service_name = NULL;
         }
-        
-        memcpy(result, data, length);
-        result[length] = '\0';
     }
     
-    return result;
+out:
+    return major_status;
 }
 
 
