@@ -20,12 +20,12 @@
  * @author Sam Lang, Sam Meder
  * 
  * $RCSfile: init_sec_context.c,v $
- * $Revision: 1.37 $
- * $Date: 2008/01/18 19:16:36 $
+ * $Revision: 1.39 $
+ * $Date: 2009/04/01 19:01:04 $
  */
 #endif
 
-static char *rcsid = "$Id: init_sec_context.c,v 1.37 2008/01/18 19:16:36 bester Exp $";
+static char *rcsid = "$Id: init_sec_context.c,v 1.39 2009/04/01 19:01:04 basney Exp $";
 
 #include "gssapi_openssl.h"
 #include "globus_i_gsi_gss_utils.h"
@@ -270,31 +270,33 @@ GSS_CALLCONV gss_init_sec_context(
                 if(g_OID_equal(((gss_name_desc*)  target_name)->name_oid,
                                GSS_C_NT_HOSTBASED_SERVICE))
                 {
-                    expected_name = globus_i_gsi_gssapi_get_hostname(
-                        (gss_name_desc*)  target_name);
-                    actual_name = globus_i_gsi_gssapi_get_hostname(
-                        (gss_name_desc*)
-                        context->peer_cred_handle->globusid);
-                    if(actual_name == NULL)
-                    {
-                        actual_name = "unknown";
-                    }
                     GLOBUS_GSI_GSSAPI_ERROR_RESULT(
                         minor_status,
                         GLOBUS_GSI_GSSAPI_ERROR_AUTHZ_DENIED,
-                        (_GGSL("The name of the remote host (%s), and the expected "
-                               "name for the remote host (%s) do not match. This happens when the name in the host certificate does not match the information obtained from DNS and is often a DNS configuration problem."),
-                         actual_name, expected_name));
-                    free(actual_name);
-                    free(expected_name);                    
+                        (_GGSL("The expected name for the remote host (%s%s%s) does not match the authenticated "
+                               "name of the remote host (%s%s%s). This happens when the name in the host certificate does not match the information obtained from DNS and is often a DNS configuration problem."),
+                         target_name->service_name
+                            ? target_name->service_name : "",
+                         target_name->service_name
+                            ? "@" : "",
+                         target_name->host_name
+                            ? target_name->host_name : "unknown",
+                         context->peer_cred_handle->globusid->service_name 
+                            ? context->peer_cred_handle->globusid->service_name
+                            : "",
+                         context->peer_cred_handle->globusid->service_name 
+                            ? "@" : "",
+                         context->peer_cred_handle->globusid->host_name 
+                            ?  context->peer_cred_handle->globusid->host_name 
+                            : "unknown"));
                 }
                 else
                 { 
-                    expected_name = X509_NAME_oneline(
-                        ((gss_name_desc*)  target_name)->x509n, NULL, 0);
-                    actual_name = X509_NAME_oneline(
+                    expected_name =
+                        ((gss_name_desc*)  target_name)->x509n_oneline;
+                    actual_name = 
                         ((gss_name_desc*)
-                         context->peer_cred_handle->globusid)->x509n, NULL, 0);
+                         context->peer_cred_handle->globusid)->x509n_oneline;
                     
                     GLOBUS_GSI_GSSAPI_ERROR_RESULT(
                         minor_status,
@@ -302,8 +304,6 @@ GSS_CALLCONV gss_init_sec_context(
                         (_GGSL("The name of the remote entity (%s), and the expected "
                                "name for the remote entity (%s) do not match"),
                          actual_name, expected_name));
-                    OPENSSL_free(actual_name);
-                    OPENSSL_free(expected_name);
                 }
 
                 major_status = GSS_S_UNAUTHORIZED;
